#if HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "general/mutexlock.hh"
#include "general/unordered_map.hh"

#include "genericAPI/Symbols.hh"

namespace GenericAPI
{
  //---------------------------------------------------------------------
  // The API name
  //---------------------------------------------------------------------
  static MutexLock::lock_type&
  api_name_baton( )
  {
    static MutexLock::lock_type baton = MutexLock::Initialize( );

    return baton;
  }

  static std::string&
  api_name( )
  {
    static std::string api( "" );
    return api;
  }

  void
  APINameGet( std::string& Name )
  {
    MutexLock	l( api_name_baton( ) );

    Name = api_name( );
  }

  void
  APINameSet( const std::string& Name )
  {
    MutexLock	l( api_name_baton( ) );

    api_name( ) = Name;
  }

  //---------------------------------------------------------------------
  // The email notify lists
  //---------------------------------------------------------------------
  typedef General::unordered_map< std::string, std::string >
  email_notify_container_type;

  static MutexLock::lock_type EMailNotifyBaton = MutexLock::Initialize( );
  static email_notify_container_type EMailNotify;

  void
  EMailNotifyGet( const std::string& Name,
		  std::string& Value )
  {
    MutexLock	l( EMailNotifyBaton );
    
    email_notify_container_type::const_iterator
      pos = EMailNotify.find( Name );
    if ( pos == EMailNotify.end( ) )
    {
      Value = EMailNotify[ "default" ];
    }
    else
    {
      Value = pos->second;
    }
  }

  void
  EMailNotifySet( const std::string& Name,
		  const std::string& Value )
  {
    MutexLock	l( EMailNotifyBaton );

    EMailNotify[ Name ] = Value;
  }

  namespace Symbols
  {
#define INIT(x,y)\
    MutexLock::lock_type symbol< x::value_type >::m_baton =  MutexLock::Initialize( ); \
	x::value_type symbol< x::value_type >::m_value = y

#if 1
#else
    INIT(HTTPUrl,"");
#endif

    // INIT(RUN_CODE,"StandAlone");

#undef INIT

    SYMBOL_CLASS_INIT(HTTP_URL,"");
    SYMBOL_CLASS_INIT(LDAS_ARCHIVE_DIR,"");
    SYMBOL_CLASS_INIT(LDAS_LOG_DIR,"");
    SYMBOL_CLASS_INIT(LDAS_MANAGER_HOST,"");
    SYMBOL_CLASS_INIT(LDAS_MANAGER_KEY,"");
    SYMBOL_CLASS_INIT(LDAS_MANAGER_PORT_EMERGENCY,0);
    SYMBOL_CLASS_INIT(LDAS_SYSTEM,"StandAlone");
    SYMBOL_CLASS_INIT(RUN_CODE,"StandAlone");
  }
}
